<?php

namespace Drupal\component_schema\Template;

use Drupal\component_schema\Component\TypedComponentManager;
use Drupal\Core\Template\TwigEnvironment;

/**
 * A class providing component Twig extensions.
 */
class ComponentTwigExtension extends \Twig_Extension {

  /**
   * The Twig environment.
   *
   * @var \Drupal\Core\Template\TwigEnvironment
   */
  protected $twig;

  /**
   * The typed component manager.
   *
   * @var \Drupal\component_schema\Component\TypedComponentManager
   */
  protected $typedComponentManager;

  /**
   * Constructs a new ComponentTwigExtension.
   *
   * @param \Drupal\Core\Template\TwigEnvironment $twig
   *   The Twig environment.
   * @param \Drupal\component_schema\Component\TypedComponentManager $typed_component_manager
   *   The typed component manager.
   */
  public function __construct(TwigEnvironment $twig, TypedComponentManager $typed_component_manager) {
    $this->twig = $twig;
    $this->typedComponentManager = $typed_component_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new \Twig_SimpleFunction('process_component', [
        $this,
        'processComponent',
      ], [
        'needs_context' => TRUE,
      ]),
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'component_schema';
  }

  /**
   * Processes the variables passed to a component.
   *
   * @param array $template_variables
   *   The variables available to a component template.
   *
   * @return array|bool|NULL
   *   If in debug mode, an array of errors or TRUE if no errors or FALSE if
   *   the component schema is missing.. Otherwise, NULL, since the work was
   *   done in the argument passed by reference.
   */
  public function processComponent(&$template_variables) {
    assert(!empty($template_variables['component_type']), 'The component_type variable is set');

    $component_type = $template_variables['component_type'];

    // Only pass relevant data for processing and validation.
    $definition = $this->typedComponentManager->getDefinition($component_type);
    $process_variables = array_intersect_key($template_variables, $definition['mapping']);

    // Process the variables.
    $schema = $this->typedComponentManager->createFromNameAndData($component_type, $process_variables);

    $process_variables = $schema->getValue();

    $return = NULL;

    // @todo Validate the schema data.
    // @see https://www.drupal.org/project/component_schema/issues/3154370

    $template_variables = array_merge($template_variables, $process_variables);

    return $return;
  }

}
